@@ -1,6 +1,8 @@
 extern crate nyx_space as nyx;
 
 mod b_plane;
+mod finite_burns;
 mod multi_oe;
 mod multi_oe_vnc;
+mod opti_levenberg;
 mod single_oe;
