@@ -1,6 +1,6 @@
 extern crate nyx_space as nyx;
 
-use nyx::md::targeter::*;
+use nyx::md::optimizer::*;
 use nyx::md::ui::*;
 
 // Semi major axis
@@ -25,15 +25,15 @@ fn tgt_sma_from_apo() {
     let spacecraft = Spacecraft::from_srp_defaults(xi_orig, 100.0, 0.0);
 
     let dynamics = SpacecraftDynamics::new(OrbitalDynamics::two_body());
-    let setup = Propagator::default(dynamics);
+    let setup = Propagator::default_dp78(dynamics);
 
     // Try to increase SMA
     let xf_desired_sma = 8_100.0;
 
     // Define the objective
-    let objectives = vec![Objective::new(StateParameter::SMA, xf_desired_sma)];
+    let objectives = [Objective::new(StateParameter::SMA, xf_desired_sma)];
 
-    let tgt = Targeter::delta_v(&setup, objectives);
+    let tgt = Optimizer::delta_v(&setup, objectives);
 
     println!("{}", tgt);
 
@@ -52,9 +52,10 @@ fn tgt_sma_from_apo() {
 
     // Check that the solutions nearly match
     println!(
-        "GMAT validation - tgt_sma_from_apo: Δv = {:.3} m/s\terr = {:.6} m/s",
+        "GMAT validation - tgt_sma_from_apo: Δv = {:.3} m/s\terr = {:.6} m/s (better = {})",
         solution_fd.correction.norm() * 1e3,
-        (solution_fd.correction.norm() - gmat_sol).abs() * 1e3
+        (solution_fd.correction.norm() - gmat_sol).abs() * 1e3,
+        solution_fd.correction.norm() < gmat_sol
     );
 }
 
@@ -81,15 +82,15 @@ fn tgt_sma_from_peri_fd() {
         &[Bodies::Luna, Bodies::Sun, Bodies::JupiterBarycenter],
         cosm,
     ));
-    let setup = Propagator::default(dynamics);
+    let setup = Propagator::default_dp78(dynamics);
 
     // Try to increase SMA
     let xf_desired_sma = 8_100.0;
 
     // Define the objective
-    let objectives = vec![Objective::new(StateParameter::SMA, xf_desired_sma)];
+    let objectives = [Objective::new(StateParameter::SMA, xf_desired_sma)];
 
-    let tgt = Targeter::delta_v(&setup, objectives);
+    let tgt = Optimizer::delta_v(&setup, objectives);
 
     println!("{}", tgt);
 
@@ -101,9 +102,10 @@ fn tgt_sma_from_peri_fd() {
 
     let gmat_sol = 0.03550369448069638;
     println!(
-        "GMAT validation - tgt_sma_from_peri: Δv = {:.3} m/s\terr = {:.6} m/s",
+        "GMAT validation - tgt_sma_from_peri: Δv = {:.3} m/s\terr = {:.6} m/s (better = {})",
         solution_fd.correction.norm() * 1e3,
-        (solution_fd.correction.norm() - gmat_sol).abs() * 1e3
+        (solution_fd.correction.norm() - gmat_sol).abs() * 1e3,
+        solution_fd.correction.norm() < gmat_sol
     );
     // GMAT validation
     assert!(
@@ -135,15 +137,15 @@ fn tgt_hd_sma_from_peri() {
         &[Bodies::Luna, Bodies::Sun, Bodies::JupiterBarycenter],
         cosm,
     ));
-    let setup = Propagator::default(dynamics);
+    let setup = Propagator::default_dp78(dynamics);
 
     // Try to increase SMA
     let xf_desired_sma = 8_100.0;
 
     // Define the objective
-    let objectives = vec![Objective::new(StateParameter::SMA, xf_desired_sma)];
+    let objectives = [Objective::new(StateParameter::SMA, xf_desired_sma)];
 
-    let mut tgt = Targeter::delta_v(&setup, objectives);
+    let mut tgt = Optimizer::delta_v(&setup, objectives);
     tgt.iterations = 5;
 
     println!("{}", tgt);
@@ -156,9 +158,10 @@ fn tgt_hd_sma_from_peri() {
 
     let gmat_sol = 0.03550369448069638;
     println!(
-        "GMAT validation - tgt_sma_from_peri: Δv = {:.3} m/s\terr = {:.6} m/s",
+        "GMAT validation - tgt_sma_from_peri: Δv = {:.3} m/s\terr = {:.6} m/s (better = {})",
         solution_fd.correction.norm() * 1e3,
-        (solution_fd.correction.norm() - gmat_sol).abs() * 1e3
+        (solution_fd.correction.norm() - gmat_sol).abs() * 1e3,
+        solution_fd.correction.norm() < gmat_sol
     );
     // GMAT validation
     assert!(
@@ -181,7 +184,7 @@ fn orbit_stm_chk() {
     let xi_orig = Orbit::keplerian(8_000.0, 0.2, 30.0, 60.0, 60.0, 0.0, orig_dt, eme2k);
 
     // let target_delta_t: Duration = xi_orig.period() / 2.0;
-    let target_delta_t = 100.0 * TimeUnit::Second;
+    let target_delta_t = 100.0 * Unit::Second;
 
     println!("Period: {} s", xi_orig.period().in_seconds() / 2.0);
 
@@ -191,7 +194,7 @@ fn orbit_stm_chk() {
         &[Bodies::Luna, Bodies::Sun, Bodies::JupiterBarycenter],
         cosm,
     );
-    let setup = Propagator::default(dynamics);
+    let setup = Propagator::default_dp78(dynamics);
     let mut prop_instance = setup.with(xi_orig.with_stm());
 
     let achievement_epoch = orig_dt + target_delta_t;
@@ -239,13 +242,13 @@ fn tgt_ecc_from_apo() {
     let spacecraft = Spacecraft::from_srp_defaults(xi_orig, 100.0, 0.0);
 
     let dynamics = SpacecraftDynamics::new(OrbitalDynamics::two_body());
-    let setup = Propagator::default(dynamics);
+    let setup = Propagator::default_dp78(dynamics);
 
     let xf_desired_ecc = 0.4;
 
-    let tgt = Targeter::new(
+    let tgt = Optimizer::new(
         &setup,
-        vec![
+        [
             Variable {
                 component: Vary::VelocityX,
                 max_step: 5.0,
@@ -262,7 +265,7 @@ fn tgt_ecc_from_apo() {
                 ..Default::default()
             },
         ],
-        vec![Objective::new(StateParameter::Eccentricity, xf_desired_ecc)],
+        [Objective::new(StateParameter::Eccentricity, xf_desired_ecc)],
     );
 
     println!("{}", tgt);
@@ -275,9 +278,10 @@ fn tgt_ecc_from_apo() {
 
     let gmat_sol = 0.7721483022815125;
     println!(
-        "GMAT validation - tgt_ecc_from_apo: Δv = {:.3} m/s\terr = {:.6} m/s",
+        "GMAT validation - tgt_ecc_from_apo: Δv = {:.3} m/s\terr = {:.6} m/s (better = {})",
         solution_fd.correction.norm() * 1e3,
-        (solution_fd.correction.norm() - gmat_sol).abs() * 1e3
+        (solution_fd.correction.norm() - gmat_sol).abs() * 1e3,
+        solution_fd.correction.norm() < gmat_sol
     );
     // GMAT validation
     assert!(
@@ -309,13 +313,13 @@ fn tgt_ecc_from_peri() {
         &[Bodies::Luna, Bodies::Sun, Bodies::JupiterBarycenter],
         cosm,
     ));
-    let setup = Propagator::default(dynamics);
+    let setup = Propagator::default_dp78(dynamics);
 
     let xf_desired_ecc = 0.4;
 
-    let tgt = Targeter::new(
+    let tgt = Optimizer::new(
         &setup,
-        vec![
+        [
             Variable {
                 component: Vary::VelocityX,
                 max_step: 5.0,
@@ -332,7 +336,7 @@ fn tgt_ecc_from_peri() {
                 ..Default::default()
             },
         ],
-        vec![Objective::new(StateParameter::Eccentricity, xf_desired_ecc)],
+        [Objective::new(StateParameter::Eccentricity, xf_desired_ecc)],
     );
 
     println!("{}", tgt);
@@ -345,9 +349,10 @@ fn tgt_ecc_from_peri() {
 
     let gmat_sol = 0.6926746704643234;
     println!(
-        "GMAT validation - tgt_ecc_from_peri: Δv = {:.3} m/s\terr = {:.6} m/s",
+        "GMAT validation - tgt_ecc_from_peri: Δv = {:.3} m/s\terr = {:.6} m/s (better = {})",
         solution_fd.correction.norm() * 1e3,
-        (solution_fd.correction.norm() - gmat_sol).abs() * 1e3
+        (solution_fd.correction.norm() - gmat_sol).abs() * 1e3,
+        solution_fd.correction.norm() < gmat_sol
     );
     // GMAT validation
     assert!(
@@ -377,14 +382,14 @@ fn tgt_raan_from_apo() {
     let spacecraft = Spacecraft::from_srp_defaults(xi_orig, 100.0, 0.0);
 
     let dynamics = SpacecraftDynamics::new(OrbitalDynamics::two_body());
-    let setup = Propagator::default(dynamics);
+    let setup = Propagator::default_dp78(dynamics);
 
     let xf_desired_raan = 65.0;
 
     // Define the objective
-    let objectives = vec![Objective::new(StateParameter::RAAN, xf_desired_raan)];
+    let objectives = [Objective::new(StateParameter::RAAN, xf_desired_raan)];
 
-    let tgt = Targeter::delta_v(&setup, objectives);
+    let tgt = Optimizer::delta_v(&setup, objectives);
 
     println!("{}", tgt);
 
@@ -396,9 +401,10 @@ fn tgt_raan_from_apo() {
 
     let gmat_sol = 0.30344716711198855;
     println!(
-        "GMAT validation - tgt_raan_from_apo: Δv = {:.3} m/s\terr = {:.6} m/s",
+        "GMAT validation - tgt_raan_from_apo: Δv = {:.3} m/s\terr = {:.6} m/s (better = {})",
         solution_fd.correction.norm() * 1e3,
-        (solution_fd.correction.norm() - gmat_sol).abs() * 1e3
+        (solution_fd.correction.norm() - gmat_sol).abs() * 1e3,
+        solution_fd.correction.norm() < gmat_sol
     );
     // GMAT validation
     assert!(
@@ -427,16 +433,16 @@ fn tgt_raan_from_peri() {
     let spacecraft = Spacecraft::from_srp_defaults(xi_orig, 100.0, 0.0);
 
     let dynamics = SpacecraftDynamics::new(OrbitalDynamics::two_body());
-    let setup = Propagator::default(dynamics);
+    let setup = Propagator::default_dp78(dynamics);
 
     let xf_desired_raan = 65.0;
 
     // Define the objective
-    let objectives = vec![Objective::new(StateParameter::RAAN, xf_desired_raan)];
+    let objectives = [Objective::new(StateParameter::RAAN, xf_desired_raan)];
 
-    let tgt = Targeter::new(
+    let tgt = Optimizer::new(
         &setup,
-        vec![
+        [
             Variable {
                 component: Vary::VelocityX,
                 max_step: 0.5,
@@ -466,9 +472,10 @@ fn tgt_raan_from_peri() {
 
     let gmat_sol = 0.45110541873478793;
     println!(
-        "GMAT validation - tgt_raan_from_peri: Δv = {:.3} m/s\terr = {:.6} m/s",
+        "GMAT validation - tgt_raan_from_peri: Δv = {:.3} m/s\terr = {:.6} m/s (better = {})",
         solution_fd.correction.norm() * 1e3,
-        (solution_fd.correction.norm() - gmat_sol).abs() * 1e3
+        (solution_fd.correction.norm() - gmat_sol).abs() * 1e3,
+        solution_fd.correction.norm() < gmat_sol
     );
     // GMAT validation
     assert!(
@@ -498,14 +505,14 @@ fn tgt_aop_from_apo() {
     let spacecraft = Spacecraft::from_srp_defaults(xi_orig, 100.0, 0.0);
 
     let dynamics = SpacecraftDynamics::new(OrbitalDynamics::two_body());
-    let setup = Propagator::default(dynamics);
+    let setup = Propagator::default_dp78(dynamics);
 
     let xf_desired_aop = 65.0;
 
     // Define the objective
-    let objectives = vec![Objective::new(StateParameter::AoP, xf_desired_aop)];
+    let objectives = [Objective::new(StateParameter::AoP, xf_desired_aop)];
 
-    let tgt = Targeter::delta_v(&setup, objectives);
+    let tgt = Optimizer::delta_v(&setup, objectives);
 
     println!("{}", tgt);
 
@@ -517,9 +524,10 @@ fn tgt_aop_from_apo() {
 
     let gmat_sol = 0.11772316331182386;
     println!(
-        "GMAT validation - tgt_aop_from_apo: Δv = {:.3} m/s\terr = {:.6} m/s",
+        "GMAT validation - tgt_aop_from_apo: Δv = {:.3} m/s\terr = {:.6} m/s (better = {})",
         solution_fd.correction.norm() * 1e3,
-        (solution_fd.correction.norm() - gmat_sol).abs() * 1e3
+        (solution_fd.correction.norm() - gmat_sol).abs() * 1e3,
+        solution_fd.correction.norm() < gmat_sol
     );
     // GMAT validation
     assert!(
@@ -548,14 +556,14 @@ fn tgt_aop_from_peri() {
     let spacecraft = Spacecraft::from_srp_defaults(xi_orig, 100.0, 0.0);
 
     let dynamics = SpacecraftDynamics::new(OrbitalDynamics::two_body());
-    let setup = Propagator::default(dynamics);
+    let setup = Propagator::default_dp78(dynamics);
 
     let xf_desired_aop = 65.0;
 
     // Define the objective
-    let objectives = vec![Objective::new(StateParameter::AoP, xf_desired_aop)];
+    let objectives = [Objective::new(StateParameter::AoP, xf_desired_aop)];
 
-    let tgt = Targeter::delta_v(&setup, objectives);
+    let tgt = Optimizer::delta_v(&setup, objectives);
 
     println!("{}", tgt);
 
@@ -567,9 +575,10 @@ fn tgt_aop_from_peri() {
 
     let gmat_sol = 0.12197875695918228;
     println!(
-        "GMAT validation - tgt_aop_from_peri: Δv = {:.3} m/s\terr = {:.6} m/s",
+        "GMAT validation - tgt_aop_from_peri: Δv = {:.3} m/s\terr = {:.6} m/s (better = {})",
         solution_fd.correction.norm() * 1e3,
-        (solution_fd.correction.norm() - gmat_sol).abs() * 1e3
+        (solution_fd.correction.norm() - gmat_sol).abs() * 1e3,
+        solution_fd.correction.norm() < gmat_sol
     );
     // GMAT validation
     assert!(
