@@ -1,12 +1,14 @@
 extern crate nyx_space as nyx;
 extern crate pretty_env_logger;
 
-use hifitime::TimeUnitHelper;
+use hifitime::TimeUnits;
 use nyx::cosmic::{Cosm, GuidanceMode, Orbit, Spacecraft};
-use nyx::dynamics::guidance::{Achieve, GuidanceLaw, Ruggiero, Thruster};
+use nyx::dynamics::guidance::{GuidanceLaw, Ruggiero, Thruster};
 use nyx::dynamics::{OrbitalDynamics, SpacecraftDynamics};
+use nyx::md::ui::Objective;
+use nyx::md::StateParameter;
 use nyx::propagators::*;
-use nyx::time::{Epoch, TimeSeries, TimeUnit};
+use nyx::time::{Epoch, TimeSeries, Unit};
 use nyx::State;
 use std::sync::mpsc::channel;
 
@@ -28,13 +30,13 @@ fn traj_ephem() {
     let setup = Propagator::default(OrbitalDynamics::two_body());
     let mut prop = setup.with(start_state);
     // The trajectory must always be generated on its own thread, no need to worry about it ;-)
-    let (end_state, ephem) = prop.for_duration_with_traj(31 * TimeUnit::Day).unwrap();
+    let (end_state, ephem) = prop.for_duration_with_traj(31 * Unit::Day).unwrap();
 
     // Example of iterating through the trajectory.
     // Note how we receive a fully blown Orbit, so we can manipulate it exactly like a normal state.
     let mut sum_sma = 0.0;
     let mut cnt = 0.0;
-    for state in ephem.every(1 * TimeUnit::Day) {
+    for state in ephem.every(1 * Unit::Day) {
         cnt += 1.0;
         sum_sma += state.sma()
     }
@@ -51,7 +53,7 @@ fn traj_ephem() {
     assert_eq!(ephem.last(), end_state, "Wrong final state");
     assert!(ephem.last().stm().is_err(), "STM is set!");
     assert!(
-        ephem.at(end_state.dt + 1 * TimeUnit::Nanosecond).is_err(),
+        ephem.at(end_state.dt + 1 * Unit::Nanosecond).is_err(),
         "Expected to be outside of interpolation window!"
     );
 
@@ -63,7 +65,7 @@ fn traj_ephem() {
     std::thread::spawn(move || {
         setup
             .with(start_state)
-            .for_duration_with_channel(31 * TimeUnit::Day, tx)
+            .for_duration_with_channel(31 * Unit::Day, tx)
             .unwrap();
     });
 
@@ -131,7 +133,7 @@ fn traj_ephem() {
     let mut max_pos_err = (eval_state.radius() - conv_state.radius()).norm();
     let mut max_vel_err = (eval_state.velocity() - conv_state.velocity()).norm();
 
-    for conv_state in ephem_back_to_earth.every(5 * TimeUnit::Minute) {
+    for conv_state in ephem_back_to_earth.every(5 * Unit::Minute) {
         let eval_state = ephem.at(conv_state.dt).unwrap();
 
         let pos_err = (eval_state.radius() - conv_state.radius()).norm();
@@ -186,29 +188,30 @@ fn traj_spacecraft() {
     let orbit = Orbit::keplerian(sma, 5e-5, 5e-3, 0.0, 178.0, 0.0, start_dt, eme2k);
 
     // Define the objectives and the control law
-    let objectives = vec![Achieve::Aop {
-        target: 183.0,
-        tol: 5e-3,
-    }];
+    let objectives = &[Objective::within_tolerance(
+        StateParameter::AoP,
+        183.0,
+        5e-3,
+    )];
 
-    let ruggiero_ctrl = Ruggiero::new(objectives, orbit);
+    let ruggiero_ctrl = Ruggiero::new(objectives, orbit).unwrap();
 
     // Build the spacecraft state
     let fuel_mass = 67.0;
     let dry_mass = 300.0;
     // Define the thruster
     let lowt = Thruster {
-        thrust: 89e-3,
-        isp: 1650.0,
+        thrust_N: 89e-3,
+        isp_s: 1650.0,
     };
     let start_state =
         Spacecraft::from_thruster(orbit, dry_mass, fuel_mass, lowt, GuidanceMode::Thrust);
 
     let sc_dynamics =
-        SpacecraftDynamics::from_ctrl(OrbitalDynamics::two_body(), ruggiero_ctrl.clone());
+        SpacecraftDynamics::from_guidance_law(OrbitalDynamics::two_body(), ruggiero_ctrl.clone());
 
     let setup = Propagator::default(sc_dynamics);
-    let prop_time = 44 * TimeUnit::Minute + 10 * TimeUnit::Second;
+    let prop_time = 44 * Unit::Minute + 10 * Unit::Second;
     let mut prop = setup.with(start_state);
     let (end_state, traj) = prop.for_duration_with_traj(prop_time).unwrap();
 
@@ -220,30 +223,32 @@ fn traj_spacecraft() {
     // Note: _Because_ we need to use the trajectory below, we'll be cloning the trajectory
     // If you don't need the trajectory after you've iterated over it, don't clone it (rustc will tell you that).
 
-    for sc_state in traj.every(1 * TimeUnit::Day) {
+    for mut sc_state in traj.every(1 * Unit::Day) {
         // We need to evaluate the mode of this state because the trajectory does not store discrete information
-        let mode_then = ruggiero_ctrl.next(&sc_state);
-        if mode_then != prev_mode {
+        ruggiero_ctrl.next(&mut sc_state);
+        if sc_state.mode() != prev_mode {
             println!(
                 "Mode changed from {:?} to {:?} @ {}",
                 prev_mode,
-                mode_then,
+                sc_state.mode(),
                 sc_state.epoch()
             );
-            prev_mode = mode_then;
+            prev_mode = sc_state.mode();
         }
     }
 
-    for epoch in TimeSeries::inclusive(start_dt, start_dt + prop_time, 1 * TimeUnit::Day) {
+    for epoch in TimeSeries::inclusive(start_dt, start_dt + prop_time, 1 * Unit::Day) {
         // Note: the `evaluate` function will return a Result which prevents a panic if you request something out of the ephemeris
-        let sc_state = traj.at(epoch).unwrap();
-        let mode_then = ruggiero_ctrl.next(&sc_state);
-        if mode_then != prev_mode {
+        let mut sc_state = traj.at(epoch).unwrap();
+        ruggiero_ctrl.next(&mut sc_state);
+        if sc_state.mode() != prev_mode {
             println!(
                 "Mode changed from {:?} to {:?} @ {}",
-                prev_mode, mode_then, epoch
+                prev_mode,
+                sc_state.mode(),
+                epoch
             );
-            prev_mode = mode_then;
+            prev_mode = sc_state.mode();
         }
     }
 
@@ -253,8 +258,7 @@ fn traj_spacecraft() {
     assert_eq!(traj.first(), start_state, "Wrong initial state");
     assert_eq!(traj.last(), end_state, "Wrong final state");
     assert!(
-        traj.at(end_state.epoch() + 1 * TimeUnit::Nanosecond)
-            .is_err(),
+        traj.at(end_state.epoch() + 1 * Unit::Nanosecond).is_err(),
         "Expected to be outside of interpolation window!"
     );
 
@@ -343,7 +347,7 @@ fn traj_spacecraft() {
     let mut max_pos_err = (eval_state.orbit.radius() - conv_state.orbit.radius()).norm();
     let mut max_vel_err = (eval_state.orbit.velocity() - conv_state.orbit.velocity()).norm();
 
-    for conv_state in ephem_back_to_earth.every(5 * TimeUnit::Minute) {
+    for conv_state in ephem_back_to_earth.every(5 * Unit::Minute) {
         let eval_state = traj.at(conv_state.epoch()).unwrap();
 
         let pos_err = (eval_state.orbit.radius() - conv_state.orbit.radius()).norm();
@@ -356,7 +360,7 @@ fn traj_spacecraft() {
         }
     }
     println!(
-        "[traj_ephem] Maximum interpolation error after double conversion: pos: {:.2e} m\t\tvel: {:.2e} m/s",
+        "[traj_spacecraft] Maximum interpolation error after double conversion: pos: {:.2e} m\t\tvel: {:.2e} m/s",
         max_pos_err * 1e3,
         max_vel_err * 1e3,
     );
@@ -374,11 +378,9 @@ fn traj_spacecraft() {
     );
 }
 
-#[ignore]
 #[allow(clippy::identity_op)]
 #[test]
 fn traj_ephem_backward() {
-    // BROKEN: cf. https://gitlab.com/chrisrabotin/nyx/-/issues/190
     // Test that we can correctly interpolate a spacecraft orbit
     let cosm = Cosm::de438();
     let eme2k = cosm.frame("EME2000");
@@ -390,16 +392,16 @@ fn traj_ephem_backward() {
 
     let setup = Propagator::default(OrbitalDynamics::two_body());
     let mut prop = setup.with(start_state);
-    let (end_state, ephem) = prop.for_duration_with_traj(-31 * TimeUnit::Day).unwrap();
+    let (end_state, ephem) = prop.for_duration_with_traj(-31 * Unit::Day).unwrap();
 
     // Example of iterating through the trajectory.
     // Note how we receive a fully blown Orbit, so we can manipulate it exactly like a normal state.
     let mut sum_sma = 0.0;
     let mut cnt = 0.0;
-    for epoch in TimeSeries::inclusive(start_dt - 31 * TimeUnit::Day, start_dt, 1 * TimeUnit::Day) {
+    for epoch in TimeSeries::inclusive(start_dt - 31 * Unit::Day, start_dt, 1 * Unit::Day) {
         cnt += 1.0;
         // Note: the `evaluate` function will return a Result which prevents a panic if you request something out of the ephemeris
-        // let state = ephem.at(epoch + 17 * TimeUnit::Second).unwrap();
+        // let state = ephem.at(epoch + 17 * Unit::Second).unwrap();
         // sum_sma += state.sma();
         match ephem.at(epoch) {
             Ok(state) => sum_sma += state.sma(),
@@ -411,16 +413,22 @@ fn traj_ephem_backward() {
     // === Below is the validation of the ephemeris == //
 
     assert_eq!(
-        ephem.segments.len(),
-        1010,
-        "Wrong number of expected segments"
+        ephem.first(),
+        start_state,
+        "Wrong initial state\nGot:  {}\nWant: {}",
+        ephem.first(),
+        start_state
     );
-
-    assert_eq!(ephem.first(), start_state, "Wrong initial state");
-    assert_eq!(ephem.last(), end_state, "Wrong final state");
-    assert!(ephem.last().stm().unwrap().norm() > 0.0, "STM is not set!");
+    assert_eq!(
+        ephem.last(),
+        end_state,
+        "Wrong final state\nGot:  {}\nWant: {}",
+        ephem.last(),
+        end_state
+    );
+    assert!(ephem.last().stm().is_err(), "STM is set!");
     assert!(
-        ephem.at(end_state.dt + 1 * TimeUnit::Nanosecond).is_err(),
+        ephem.at(end_state.dt - 1 * Unit::Nanosecond).is_err(),
         "Expected to be outside of interpolation window!"
     );
 
@@ -430,7 +438,7 @@ fn traj_ephem_backward() {
     std::thread::spawn(move || {
         setup
             .with(start_state)
-            .for_duration_with_channel(31 * TimeUnit::Day, tx)
+            .for_duration_with_channel(-31 * Unit::Day, tx)
             .unwrap();
     });
 
@@ -441,6 +449,8 @@ fn traj_ephem_backward() {
     let mut max_vel_err = (eval_state.velocity() - start_state.velocity()).norm();
     let mut max_err = (eval_state.as_vector().unwrap() - start_state.as_vector().unwrap()).norm();
 
+    println!("{}", ephem);
+
     while let Ok(prop_state) = rx.recv() {
         let eval_state = ephem.at(prop_state.dt).unwrap();
 
@@ -459,7 +469,7 @@ fn traj_ephem_backward() {
     }
 
     println!(
-        "[traj_ephem] Maximum interpolation error: pos: {:.2e} m\t\tvel: {:.2e} m/s\t\tfull state: {:.2e} (no unit)",
+        "[traj_ephem_backward] Maximum interpolation error: pos: {:.2e} m\t\tvel: {:.2e} m/s\t\tfull state: {:.2e} (no unit)",
         max_pos_err * 1e3,
         max_vel_err * 1e3,
         max_err
